--- res_musiconhold.c.orig	Sat Nov 18 10:46:17 2006
+++ res_musiconhold.c	Sat Nov 18 11:21:11 2006
@@ -194,8 +194,7 @@
 	}
 }
 
-
-static int ast_moh_files_next(struct ast_channel *chan) 
+static void ast_moh_files_next_index(struct ast_channel *chan)
 {
 	struct moh_files_state *state = chan->music_state;
 	int tries;
@@ -226,6 +225,14 @@
 	}
 
 	state->pos = state->pos % state->class->total_files;
+}
+
+
+static int ast_moh_files_next(struct ast_channel *chan) 
+{
+	struct moh_files_state *state = chan->music_state;
+
+	ast_moh_files_next_index(chan);
 	
 	if (!ast_openstream_full(chan, state->class->filearray[state->pos], chan->language, 1)) {
 		ast_log(LOG_WARNING, "Unable to open file '%s': %s\n", state->class->filearray[state->pos], strerror(errno));
@@ -297,6 +304,30 @@
 			/* initialize */
 			memset(state, 0, sizeof(struct moh_files_state));
 			state->class = class;
+			/*
+			 * Assign a file position now.  If we don't
+			 * moh_files_release gets called before we
+			 * start playing (I don't know why) and it
+			 * sets save_pos to pos + 1.  That makes save_pos
+			 * equal to 1.  When we finally get around to
+			 * playing the music on hold, the random flag
+			 * won't work, since ast_moh_files_next uses
+			 * the save_pos value in preference of finding
+			 * a new file if save_pos is non-zero.  Since
+			 * pos is zero right now due to the memset above,
+			 * save_pos is always 1 and we always starting
+			 * playing the first file in the dirent - which
+			 * isn't particularly random.
+			 *
+			 * By calling ast_moh_files_next_index right now,
+			 * pos will be set to a random value (if randomize
+			 * is enabled).  Then when save_pos is set to
+			 * pos + 1, save_pos is still a random value.
+			 * When ast_moh_files_next is called when we start
+			 * playing the moh, it will use the save_pos value
+			 * (it is non-zero) and we'll have random moh.
+			 */
+			ast_moh_files_next_index(chan);
 		}
 
 		state->origwfmt = chan->writeformat;
